import json
import sys
import logging
import dnf
from . import _depchase, _repodata
from .util import rpm_name_only

class ModuleRepoquery(object):

    def list_modules(self):
        module_names = _repodata.list_modules()
        if module_names:
            for name in sorted(module_names):
                print(name)

    def list_modules_for_rpm(self, pkg):
        module_names = _repodata.get_modules_for_rpm(pkg)
        if module_names:
            for name in sorted(module_names):
                print(name)

    def list_rpms_in_module(self, module, full_nevra=False):
        rpm_names = _repodata.get_rpms_in_module(module)
        if rpm_names:
            for name in sorted(rpm_names):
                if full_nevra:
                    print(name)
                else:
                    print(rpm_name_only(name))

    def list_pkg_deps(self, pkgs, module_deps, json_output=False):
        pkgs_in_modules = set()
        if module_deps:
            for module in module_deps:
                rpm_names = _repodata.get_rpms_in_module(module)
                pkgs_in_modules |= set(map(lambda x: rpm_name_only(x), rpm_names))

        pool = _depchase.make_pool("x86_64")
        if json_output:
            run_deps = _depchase.ensure_installable(pool, pkgs, full_info=True)
            result = []
            for info in run_deps:
                if info.name in pkgs_in_modules:
                    continue
                result.append({
                    'rpm': info.rpm,
                    'srpm': info.srpm,
                    'requires': info.requires,
                })
            json.dump(result, sys.stdout, indent=4, sort_keys=True)
            print()
        else:
            run_deps = _depchase.ensure_installable(pool, pkgs)
            rpm_names = run_deps - pkgs_in_modules
            if rpm_names:
                for name in rpm_names:
                    print(name)

    def list_modularized_pkgs(self, duplicate_only=False, list_modules=False):
        rpm_names = _repodata.get_rpm_reverse_lookup()
        if rpm_names:
            for name in sorted(rpm_names.keys()):
                if not (duplicate_only and not len(rpm_names[name]) > 1):
                    if list_modules:
                        print(name + "      (" + ", ".join(rpm_names[name]) + ")")
                    else:
                        print(name)

    def get_srpm_for_rpm(self, pkg):
        pool = _depchase.make_pool("x86_64")
        srpm = _depchase.get_srpm_for_rpm(pool, pkg)
        if srpm:
            print(rpm_name_only(srpm))

    def get_rpms_for_srpm(self, pkg):
        pool = _depchase.make_pool("x86_64")
        rpms = _depchase.get_rpms_for_srpms(pool, [pkg])
        for rpm in sorted(rpms):
            print(rpm)
